package javax.rmi;

import java.net.MalformedURLException;
import java.rmi.server.RMIClassLoader;
import java.util.Properties;
import org.omg.CORBA.INITIALIZE;
import java.security.AccessController;
import com.sun.corba.se.impl.orbutil.GetPropertyAction;
import java.rmi.NoSuchObjectException;
import java.rmi.RemoteException;
import java.rmi.Remote;
import javax.rmi.CORBA.PortableRemoteObjectDelegate;

public class PortableRemoteObject
{
    private static final PortableRemoteObjectDelegate proDelegate;
    private static final String PortableRemoteObjectClassKey = "javax.rmi.CORBA.PortableRemoteObjectClass";
    
    protected PortableRemoteObject() throws RemoteException {
        if (PortableRemoteObject.proDelegate != null) {
            exportObject((Remote)this);
        }
    }
    
    public static void exportObject(Remote remote) throws RemoteException {
        if (PortableRemoteObject.proDelegate != null) {
            PortableRemoteObject.proDelegate.exportObject(remote);
        }
    }
    
    public static Remote toStub(Remote remote) throws NoSuchObjectException {
        if (PortableRemoteObject.proDelegate != null) {
            return PortableRemoteObject.proDelegate.toStub(remote);
        }
        return null;
    }
    
    public static void unexportObject(Remote remote) throws NoSuchObjectException {
        if (PortableRemoteObject.proDelegate != null) {
            PortableRemoteObject.proDelegate.unexportObject(remote);
        }
    }
    
    public static Object narrow(Object o, Class clazz) throws ClassCastException {
        if (PortableRemoteObject.proDelegate != null) {
            return PortableRemoteObject.proDelegate.narrow(o, clazz);
        }
        return null;
    }
    
    public static void connect(Remote remote, Remote remote2) throws RemoteException {
        if (PortableRemoteObject.proDelegate != null) {
            PortableRemoteObject.proDelegate.connect(remote, remote2);
        }
    }
    
    private static Object createDelegate(String key) {
        String property = (String)AccessController.doPrivileged(new GetPropertyAction(key));
        if (property == null) {
            Properties orbPropertiesFile = getORBPropertiesFile();
            if (orbPropertiesFile != null) {
                property = orbPropertiesFile.getProperty(key);
            }
        }
        if (property == null) {
            return new com.sun.corba.se.impl.javax.rmi.PortableRemoteObject();
        }
        try {
            return loadDelegateClass(property).newInstance();
        }
        catch (ClassNotFoundException ex) {
            INITIALIZE o = new INITIALIZE("Cannot instantiate " + property);
            o.initCause(ex);
            throw o;
        }
        catch (Exception ex2) {
            INITIALIZE o2 = new INITIALIZE("Error while instantiating" + property);
            o2.initCause(ex2);
            throw o2;
        }
    }
    
    private static Class loadDelegateClass(String str) throws ClassNotFoundException {
        try {
            return Class.forName(str, false, Thread.currentThread().getContextClassLoader());
        }
        catch (ClassNotFoundException ex) {
            try {
                return RMIClassLoader.loadClass(str);
            }
            catch (MalformedURLException ex2) {
                throw new ClassNotFoundException("Could not load " + str + ": " + ex2.toString());
            }
        }
    }
    
    private static Properties getORBPropertiesFile() {
        return (Properties)AccessController.doPrivileged(new GetORBPropertiesFileAction());
    }
    
    static {
        proDelegate = (PortableRemoteObjectDelegate)createDelegate("javax.rmi.CORBA.PortableRemoteObjectClass");
    }
}
